from __future__ import unicode_literals

import logging

from functools import reduce
from operator import and_

from django.conf import settings
from django.template.loader import render_to_string

logger = logging.getLogger('bandit.backends.base')


class HijackBackendMixin(object):
    """
    This backend mixin intercepts outgoing messages drops them to a single
    email address.
    """

    def __init__(self, *args, **kwargs):
        self.log_only = kwargs.pop('log_only', False)
        self.log_level = kwargs.pop('log_level', logging.DEBUG)
        super(HijackBackendMixin, self).__init__(*args, **kwargs)

    def send_messages(self, email_messages):
        admins = getattr(settings, 'ADMINS', ())
        server_email = getattr(settings, 'SERVER_EMAIL', 'root@localhost')
        bandit_email = getattr(settings, 'BANDIT_EMAIL', server_email)
        whitelist_emails = set(getattr(settings, 'BANDIT_WHITELIST', ()))
        approved_emails = set([server_email, bandit_email, ] + list(whitelist_emails) +
                              [email for name, email in admins])

        def is_approved(email):
            local_part, _, domain = email.rpartition('@')
            return email in approved_emails or domain in whitelist_emails

        to_send = []
        logged_count = 0
        for message in email_messages:
            recipients = message.to + message.cc + message.bcc
            all_approved = reduce(and_, map(is_approved, recipients))
            if all_approved:
                to_send.append(message)
            else:
                context = {'message': message,
                           'previous_recipients': message.to,
                           'previous_cc': message.cc,
                           'previous_bcc': message.bcc
                           }  # included for backwards compatibility
                log_message = render_to_string("bandit/hijacked-email-log-message.txt", context)
                logger.log(self.log_level, log_message)
                if not self.log_only:
                    header = render_to_string("bandit/hijacked-email-header.txt", context)
                    message.body = header + message.body
                    message.to = [bandit_email]
                    # clear cc/bcc
                    message.cc = []
                    message.bcc = []
                    to_send.append(message)
                else:
                    # keep track of how many messages were only logged so we
                    # can report them as sent to the caller
                    logged_count += 1
        sent_count = super(HijackBackendMixin, self).send_messages(to_send) or 0
        return sent_count + logged_count


class LogOnlyBackendMixin(HijackBackendMixin):

    def __init__(self, *args, **kwargs):
        kwargs['log_only'] = True
        super(LogOnlyBackendMixin, self).__init__(*args, **kwargs)



from __future__ import unicode_literals

from seacucumber.backend import SESBackend

from bandit.backends.base import HijackBackendMixin, LogOnlyBackendMixin


class HijackSESBackend(HijackBackendMixin, SESBackend):
    """
    This backend intercepts outgoing messages drops them to a single email
    address, using the SESBackend in django-seacucumber.
    """
    pass


class LogOnlySESBackend(LogOnlyBackendMixin, SESBackend):
    """
    This backend intercepts outgoing messages and logs them, allowing
    only messages destined for ADMINS to be sent via SES.
    """
    pass



from __future__ import unicode_literals

from django.core.mail.backends.smtp import EmailBackend as SMTPBackend

from bandit.backends.base import HijackBackendMixin, LogOnlyBackendMixin


class HijackSMTPBackend(HijackBackendMixin, SMTPBackend):
    """
    This backend intercepts outgoing messages drops them to a single email
    address.
    """
    pass


class LogOnlySMTPBackend(LogOnlyBackendMixin, SMTPBackend):
    """
    This backend intercepts outgoing messages and logs them, allowing
    only messages destined for ADMINS to be sent via SMTP.
    """
    pass



import os
from setuptools import setup, find_packages


def read_file(filename):
    """Read a file into a string"""
    path = os.path.abspath(os.path.dirname(__file__))
    filepath = os.path.join(path, filename)
    try:
        return open(filepath).read()
    except IOError:
        return ''


setup(
    name='django-email-bandit',
    version=__import__('bandit').__version__,
    author='Caktus Consulting Group',
    author_email='solutions@caktusgroup.com',
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/caktus/django-email-bandit',
    license='BSD',
    description=' '.join(__import__('bandit').__doc__.splitlines()).strip(),
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ),
    long_description=read_file('README.rst'),
    test_suite="runtests.runtests",
    zip_safe=False,
)



from __future__ import unicode_literals

import asyncore
import smtpd
import threading
from email import message_from_string
try:
    # Python 3
    from email.utils import parseaddr
except ImportError:
    # Python 2
    from email.Utils import parseaddr

from django.conf import settings
from django.core.mail import get_connection, EmailMessage
from django.test import TestCase


class FakeSMTPServer(smtpd.SMTPServer, threading.Thread):
    """
    FakeSMTPServer from Django's regressiontests.mail.tests.py

    Asyncore SMTP server wrapped into a thread. Based on DummyFTPServer from:
    http://svn.python.org/view/python/branches/py3k/Lib/test/test_ftplib.py?revision=86061&view=markup
    """

    def __init__(self, *args, **kwargs):
        threading.Thread.__init__(self)
        smtpd.SMTPServer.__init__(self, *args, **kwargs)
        self._sink = []
        self.active = False
        self.active_lock = threading.Lock()
        self.sink_lock = threading.Lock()

    def process_message(self, peer, mailfrom, rcpttos, data):
        m = message_from_string(data)
        maddr = parseaddr(m.get('from'))[1]
        if mailfrom != maddr:
            return "553 '%s' != '%s'" % (mailfrom, maddr)
        self.sink_lock.acquire()
        self._sink.append(m)
        self.sink_lock.release()

    def get_sink(self):
        self.sink_lock.acquire()
        try:
            return self._sink[:]
        finally:
            self.sink_lock.release()

    def flush_sink(self):
        self.sink_lock.acquire()
        self._sink[:] = []
        self.sink_lock.release()

    def start(self):
        assert not self.active
        self.__flag = threading.Event()
        threading.Thread.start(self)
        self.__flag.wait()

    def run(self):
        self.active = True
        self.__flag.set()
        while self.active and asyncore.socket_map:
            self.active_lock.acquire()
            asyncore.loop(timeout=0.1, count=1)
            self.active_lock.release()
        asyncore.close_all()

    def stop(self):
        assert self.active
        self.active = False
        self.join()


class BaseBackendTestCase(TestCase):
    """
    Test email interception in the HijackBackend.
    """

    @classmethod
    def setUpClass(cls):
        cls.server = FakeSMTPServer(('127.0.0.1', 0), None)
        settings.EMAIL_HOST = "127.0.0.1"
        settings.EMAIL_PORT = cls.server.socket.getsockname()[1]
        cls._original_admins = settings.ADMINS
        cls._original_bandit = getattr(settings, 'BANDIT_EMAIL', '')
        settings.BANDIT_EMAIL = 'bandit@example.com'
        settings.ADMINS = (('Admin', 'admin@example.com'), )
        cls.server.start()

    @classmethod
    def tearDownClass(cls):
        cls.server.stop()
        settings.BANDIT_EMAIL = cls._original_bandit
        settings.ADMINS = cls._original_admins

    def setUp(self):
        super(BaseBackendTestCase, self).setUp()
        self.flush_mailbox()

    def tearDown(self):
        self.flush_mailbox()
        super(BaseBackendTestCase, self).tearDown()

    def get_connection(self):
        raise NotImplementedError('Must define in subclass')

    def get_mailbox_content(self):
        return self.server.get_sink()

    def flush_mailbox(self):
        self.server.flush_sink()


class HijackBackendTestCase(BaseBackendTestCase):

    def get_connection(self):
        return get_connection('bandit.backends.smtp.HijackSMTPBackend')

    def test_basic_hijack(self):
        """Emails should be redirected to send to BANDIT_EMAIL."""
        emails = [EmailMessage('Subject', 'Content', 'from@example.com', ['to@example.com'])]
        num_sent = self.get_connection().send_messages(emails)
        self.assertEqual(len(emails), num_sent)
        messages = self.get_mailbox_content()
        self.assertEqual(len(messages), num_sent)
        message = messages[0]
        self.assertEqual(message.get_all('to'), ['bandit@example.com', ])

    def test_send_to_admins(self):
        """Admin emails should not be hijacked."""
        emails = [EmailMessage('Subject', 'Content', 'from@example.com', ['admin@example.com'])]
        num_sent = self.get_connection().send_messages(emails)
        self.assertEqual(len(emails), num_sent)
        messages = self.get_mailbox_content()
        self.assertEqual(len(messages), num_sent)
        message = messages[0]
        self.assertEqual(message.get_all('to'), ['admin@example.com', ])

    def test_send_to_mixed(self):
        """Emails with mixed recipients will be hijacked."""
        emails = [EmailMessage('Subject', 'Content', 'from@example.com', ['to@example.com', 'admin@example.com'])]
        num_sent = self.get_connection().send_messages(emails)
        self.assertEqual(len(emails), num_sent)
        messages = self.get_mailbox_content()
        self.assertEqual(len(messages), num_sent)
        message = messages[0]
        self.assertEqual(message.get_all('to'), ['bandit@example.com', ])

    def test_send_multiple(self):
        """Emails with mixed recipients will be hijacked."""
        emails = [EmailMessage('Subject', 'Content', 'from@example.com', ['to@example.com']),
                  EmailMessage('Subject', 'Content', 'from@example.com', ['admin@example.com'])]
        num_sent = self.get_connection().send_messages(emails)
        self.assertEqual(len(emails), num_sent)
        messages = self.get_mailbox_content()
        self.assertEqual(len(messages), len(emails))
        message = messages[0]
        self.assertEqual(message.get_all('to'), ['bandit@example.com', ])
        message = messages[1]
        self.assertEqual(message.get_all('to'), ['admin@example.com', ])

    def test_whitelist_domain(self):
        addresses = ['foo@whitelisted.test.com', 'bar@whitelisted.test.com']
        emails = [EmailMessage( 'Subject', 'Content', 'from@example.com', addresses)]
        num_sent = self.get_connection().send_messages(emails)
        self.assertEqual(len(emails), num_sent)
        messages = self.get_mailbox_content()
        self.assertEqual(messages[0].get_all('to'), [', '.join(addresses)])


class LogOnlyBackendTestCase(BaseBackendTestCase):

    def get_connection(self):
        return get_connection('bandit.backends.smtp.LogOnlySMTPBackend')

    def test_basic_hijack(self):
        """Emails should only be logged."""
        emails = [EmailMessage('Subject', 'Content', 'from@example.com', ['to@example.com'])]
        num_sent = self.get_connection().send_messages(emails)
        self.assertEqual(len(emails), num_sent)
        messages = self.get_mailbox_content()
        self.assertEqual(len(messages), 0)

    def test_send_to_admins(self):
        """Admin emails should still be sent."""
        emails = [EmailMessage('Subject', 'Content', 'from@example.com', ['admin@example.com'])]
        num_sent = self.get_connection().send_messages(emails)
        self.assertEqual(len(emails), num_sent)
        messages = self.get_mailbox_content()
        self.assertEqual(len(messages), len(emails))
        message = messages[0]
        self.assertEqual(message.get_all('to'), ['admin@example.com', ])

    def test_send_to_mixed(self):
        """Emails with mixed recipients will only be logged."""
        emails = [EmailMessage('Subject', 'Content', 'from@example.com', ['to@example.com', 'admin@example.com'])]
        num_sent = self.get_connection().send_messages(emails)
        self.assertEqual(len(emails), num_sent)
        messages = self.get_mailbox_content()
        self.assertEqual(len(messages), 0)

    def test_send_multiple(self):
        """Only the email to the admin should be sent (the other should be logged)."""
        emails = [EmailMessage('Subject', 'Content', 'from@example.com', ['to@example.com']),
                  EmailMessage('Subject', 'Content', 'from@example.com', ['admin@example.com'])]
        num_sent = self.get_connection().send_messages(emails)
        self.assertEqual(len(emails), num_sent)
        messages = self.get_mailbox_content()
        self.assertEqual(len(messages), 1)
        message = messages[0]
        self.assertEqual(message.get_all('to'), ['admin@example.com', ])
